#include "mex.h"
#include <math.h>

#ifndef MWSIZE_MAX
typedef int mwSize;
typedef int mwIndex;
#endif

#ifndef max
#define max( a, b ) ( ((a) > (b)) ? (a) : (b) )
#endif

#ifndef min
#define min( a, b ) ( ((a) < (b)) ? (a) : (b) )
#endif

double ElectronWavelength(double Voltage) {
    static const double h = 6.626068e-34;   /*< Planck constant*/
    static const double e = 1.60217646e-19; /*< electron charge*/
    static const double m = 9.10938188e-31; /*< electron mass*/
    static const double c = 2.99792458e8;   /*< speed of light*/
    
    return h / sqrt(e * Voltage * m * (e * Voltage / (m * c*c) + 2.0 ));
}

double kValue(double qx, double qy, double *CTFmodel) {
    /* printf("CTFmodel\n lambda: %e\n cs: %e\n defocus %e\n astigma %e\n astigAn %e\n ampCont %e \n",
            CTFmodel[0], CTFmodel[1], CTFmodel[2], CTFmodel[3], CTFmodel[4], CTFmodel[5]); */
    double qsq = qx*qx + qy*qy;
    double astigTerm = CTFmodel[3] * cos(2*(atan2(qy, qx) - CTFmodel[4]));
    /*if (CTFmodel[2] > 0)
        astigTerm = -astigTerm;*/
    return (CTFmodel[1] * pow(CTFmodel[0], 3) * qsq*qsq / 2 + (CTFmodel[2] + astigTerm) * CTFmodel[0] * qsq - CTFmodel[5]);
}


/* NEW */
double kValueDerivativeQ(double qx, double qy, double *CTFmodel) {
       double qsq = qx*qx + qy*qy;
     double astigTerm = CTFmodel[3] * cos(2*(atan2(qy, qx) - CTFmodel[4]));
         /*if (CTFmodel[2] > 0)
        astigTerm = -astigTerm;*/    
   return 2 * CTFmodel[0] * sqrt(qsq) * (CTFmodel[1] * CTFmodel[0]*CTFmodel[0] * qsq + (CTFmodel[2] + astigTerm));
}
/* END OF NEW */






/* mex -setup (only once to select the compiler)
   mex -v ThonRingAverage.cpp to compile it
   matlab call: ThonRingAverage(double(sin(rr/4)), [1 100],1, [2e-12, 2e-3, -1e-6, 10e-9, 0, .1], 1, [1 2]);
   ThonRingAverage( image, radiusRange, radiusStep, CTFmodel, physicalBinningStep, sigmaRange)*/
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] ) {
    static const double RelativeGaussianTruncationSize = 3.0;
    const mwSize *sizeImg;
    mwSize sizeOut[2];
    mwSize stride_x, stride_y;
    double *img, *result, *radiusRange, *sigmaRange, *sum, *CTFmodel;
    double radiusStep, physicalBinningStep;
    int resultSize;
    double center[2], sigmaStep, pixelValue, q, qx, qy, qsq, k, astigFreeRadius, innerSqrtTerm;
    double nom; /*, astigFreeRadius;*/
    double thisRadius, gaussValue, sigma, radDiff;
    int i, hDiv2, w, x, y, stopIdx, startIdx;
    
    double lambda;
    double cs;
    double defocus;
    double astigmatism;
    double astigmatismAngle;
    double amplitudeContrastFraction;
    
    /* Input handling */
    if (nrhs !=6) {
        mexErrMsgTxt("need 6 inputs.");
    }
    if (nlhs > 1) {
        mexErrMsgTxt("Only one output argument.");
    }
    if (!mxIsDouble(prhs[0]) || mxGetNumberOfDimensions(prhs[0])!=2) {
        printf("Image has %d dimensions.\n", mxGetNumberOfDimensions(prhs[0]));
        mexErrMsgTxt("Input must be 2D double array.");
    }
    if (mxGetNumberOfElements(prhs[3])!=6) {
        mexErrMsgTxt("CTF model needs 6 parameters.");
    }
    if (mxGetNumberOfElements(prhs[1])!=2) {
        mexErrMsgTxt("radiusRange needs 2 inputs.");
    }
    if (mxGetNumberOfElements(prhs[5])!=2) {
        mexErrMsgTxt("sigmaRange needs 2 inputs.");
    }
    /* assign matlab input to c variables */
    sizeImg = mxGetDimensions(prhs[0]);
    img           = mxGetPr(prhs[0]); /*pointer to image in*/
    radiusRange   = (double*) mxGetData(prhs[1]);
    radiusStep    = mxGetScalar(prhs[2]);
    CTFmodel      = (double*) mxGetData(prhs[3]);
    physicalBinningStep =  mxGetScalar(prhs[4]);
    sigmaRange    = (double*) mxGetData(prhs[5]);
    
    /*printf("size image %d %d\n", sizeImg[0], sizeImg[1]);
    printf("radius range %f %f \n", radiusRange[0], radiusRange[1]);
    printf("sigma range %f %f \n", sigmaRange[0], sigmaRange[1]);
    printf("physicalBinningStep %f \n", physicalBinningStep);
    printf("radiusstep  %f \n", radiusStep);*/
    lambda = CTFmodel[0];
    /*lambda = ElectronWavelength(3e5);*/
    cs = CTFmodel[1];
    defocus = CTFmodel[2];
    astigmatism = CTFmodel[3];
    astigmatismAngle = CTFmodel[4];
    amplitudeContrastFraction = CTFmodel[5];
    /*printf("CTFmodel\n lambda: %e\n cs: %e\n defocus %e\n astigma %e\n astigAn %e\n ampCont %e \n",
            lambda, cs, defocus, astigmatism, astigmatismAngle, amplitudeContrastFraction);*/
    stride_x = sizeImg[0];
    stride_y = 1;
    /* printf("strides x %d, y %d\n",stride_x,stride_y);*/
    /* make the output in matlab: a 1D vector*/
    resultSize = (int) (floor((radiusRange[1] - radiusRange[0]) / radiusStep) + 1);
    /*printf("Output size %d\n", resultSize);*/
    sizeOut[0] = resultSize;
    sizeOut[1] = 1;
    plhs[0] = mxCreateNumericArray(2, sizeOut, mxDOUBLE_CLASS, mxREAL);
    result = mxGetPr(plhs[0]);
    sum = (double*) malloc(resultSize*sizeof(double));
    for (i = 0; i < resultSize; ++i) {
        result[i] = 0.0;
        sum[i] = 0.0;
    }
    
    sigmaStep = (sigmaRange[1] - sigmaRange[0]) / (resultSize - 1);
    center[0] = sizeImg[0]/2;
    center[1] = sizeImg[1]/2;
    
    hDiv2 = (int) sizeImg[1] / 2;
    w = (int) sizeImg[0];
    /*printf("hDiv2 %d, w %d\n",hDiv2,w);
    printf("sigmastep %f\n",sigmaStep);*/
    for ( y = 0; y <= hDiv2; ++y) {
    
        if (y == hDiv2)
            w  = w / 2 + 1; /*to really treat half the spectrum.*/
        for ( x = 0; x < w; ++x) {
            pixelValue = *(img + x*stride_x + y*stride_y);
            /*if (x < 10 && y <10)
               printf("pixel value [%d][%d] %f\n",x,y,pixelValue);*/
            qx = (x - center[0]) * physicalBinningStep;
            qy = -(y - center[1]) * physicalBinningStep; /* minus sign because we want math convention i.s.o. image convetion*/
            qsq = (qx* qx + qy * qy);
            q = sqrt(qsq);
            
            k = kValue(qx, qy, CTFmodel);
            astigFreeRadius = 0;
            
            /*new version
              if (model.cs > 0 && k != 0.)
                       {
                               double innerSqrtTerm = model.defocus * model.defocus + 2 * model.cs * model.lambda * k;
                               if (innerSqrtTerm < 0)
                                       innerSqrtTerm = 0; // oops, problem, non-elliptic case.
                               else
                                       innerSqrtTerm = std::sqrt(innerSqrtTerm);
             */
            /* NEW */
                 /*  double nom;
               if (model.kValueDerivativeQ(qx,qy) < 0)
               {
                   nom = -model.defocus - innerSqrtTerm; //nomMin
               }
               else
               {
                   nom = -model.defocus + innerSqrtTerm; //nomPlus
               }
               if (nom > 0)
                               {
                                       astigFreeRadius = std::sqrt(nom / model.cs) / model.lambda;
                               }
                               else
                               {
                                       assert(!"abc formula error.");
               }
/* END OF NEW */
            if (cs > 0 && k !=0) {
                innerSqrtTerm = defocus * defocus + 2 * cs * lambda * k;
                if (defocus>0)
                {innerSqrtTerm = defocus * defocus + 2 * cs * lambda * fabs(k);
                 innerSqrtTerm = sqrt(innerSqrtTerm);
                 nom  = -defocus + innerSqrtTerm;} 
                 else{                  
                        if (innerSqrtTerm < 0)
                            innerSqrtTerm = 0; /*oops, problem, non-elliptic case.*/
                        else
                            innerSqrtTerm = sqrt(innerSqrtTerm);
                        /*nomPlus = -defocus + innerSqrtTerm; 
                          nomMin  = -defocus - innerSqrtTerm; */
                         if (kValueDerivativeQ(qx, qy, CTFmodel) < 0){
                             nom  = -defocus - innerSqrtTerm;
                           /*printf("nommin %f\n",nom*1e9);*/
                        }
                        else 
                        {
                            nom  = -defocus + innerSqrtTerm; 
                           /*printf("nomplus %f\n",nom*1e9);*/
                        }}
                astigFreeRadius = sqrt(fabs(nom) / cs) / lambda;
               }
               /*if (nom >= 0)
                               {
                                       astigFreeRadius = sqrt(nom / cs) / lambda;
                               }
                               else
                               {
                                       mexErrMsgTxt("abc formula error.");
                               }                    
                } */                
            else {
                astigFreeRadius =  sqrt(fabs(k) / (lambda * fabs(defocus)));
            }
            astigFreeRadius = astigFreeRadius/physicalBinningStep;
            /*printf("nomMin %8.10f nomPlus %8.10f q %f \n",nomMin, nomPlus, q );
            printf("astigFreeRadius %f\n",astigFreeRadius);*/
            
            startIdx = max((int)(ceil(
                    (astigFreeRadius - radiusRange[0] - RelativeGaussianTruncationSize * sigmaRange[0])
                    / (radiusStep + RelativeGaussianTruncationSize * sigmaStep))), 0);
            stopIdx = min((int)(floor(
                    (astigFreeRadius - radiusRange[0] + RelativeGaussianTruncationSize * sigmaRange[0])
                    / (radiusStep - RelativeGaussianTruncationSize * sigmaStep))), resultSize - 1);
            /*if startIdx > stopIdx, automatically nothing will be done, since the radius is out of the requested range.*/
            /*if (startIdx < 1200){
               printf("%d %d\n", startIdx, stopIdx);
            }*/
            for ( i = startIdx; i <= stopIdx; ++i) {
                thisRadius = (radiusStep * i + radiusRange[0]);
                radDiff = astigFreeRadius - thisRadius;
                sigma = (radiusStep * i + radiusRange[0]) * (sigmaRange[1] - sigmaRange[0]) / radiusRange[1] + sigmaRange[0];
                gaussValue = exp(- radDiff * radDiff / (2 * sigma * sigma));
                result[i] += gaussValue * pixelValue;
                sum[i] += gaussValue;
            }
        }
    }
    for ( i = 0; i < resultSize; ++i) {
        /*printf("%d: %f \n",i,sum[i]);*/
        if (sum[i] > 0.0) {
            
            result[i] /= sum[i];
        }
    }
    free(sum);
}


