function [df, std_df, k_out, df_out] = est_df( q_0, Cs, lambda , phiamp, missedzeros, signdf)
% EST_DF Estimation of defocus based on k-trajecotry method
%
% SYNOPSIS:
% [df, std_df, k_out, df_out] = est_df( q_0, Cs, lambda, phiamp, missedzeros, signdf)
%
% PARAMETERS:
%         q_0:   sorted array of zero-crossings of the CTF [m^-1]
%          Cs:   spherical aberration [m]
%      lambda:   electron wavelength [m]
%      phiamp:   amplitude contrast vector
% missedzeros:   missed CTF zeros in detection
%      signdf:   1 underfocus, -1 overfocus, 0 undefined 
%
% OUTPUT:
%      df: estimated defocus [m]
%  std_df: uncertainty in the defocus estimation [m]
%   k_out: best k-sequence
%  df_out: defocus values that correspond to the best k-sequence 
%
% DEFAULTS:
%  signdf = 0; non specified. In case there is only one Thon ring, this
%              parameter is needed (1 for underfocus, -1 for overfocus)
%  phiamp = 0;

% (C) Copyright 2011
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
%  Milos Vulovic & Pieter Brandt


if nargin < 4; phiamp =0;end
if nargin < 5; missedzeros=[];end
if (exist('missedzeros')==1); 
    if isinteger(int8(missedzeros))
    else error(' the order of the zeros must be integer numbers')
    end
end

% q_0=zeroesPSD';
% phiamp=phiamp';

ordlastzero = length(q_0) + length(missedzeros);
orderzero = 1:ordlastzero;
orderzero(missedzeros) = [];
Ntot=max(orderzero);
N = size(q_0,2);

if ( N == 0 )   
    df = 0;
    std_df = 0;
    k_out = 0;
    
elseif ( N == 1 ) 
    df = ( lambda^3*Cs*q_0^4 + 2*signdf*orderzero - 2*phiamp) ./ (2*lambda*q_0^2);
    std_df = 0;
    k_out = signdf;
    df_est=df;
    df_est_mean=df;
    df_est_std=std_df;   
else  
    index=1:Ntot;
    n = ( 1:(2*Ntot+1) )' * ones(1,Ntot); %matrix which has n columns as the order of 2*n+1 k values (n x(2n+1)) 
    phiamp=repmat(phiamp', [1 (2*Ntot+1)]);%create the influence of amplitude contrast
    k = index' * ones(1,2*Ntot+1) ; % 2*n+1 columns with the values n (increasing order)
    k = -min(k,n'-k-1); %figure 1.9 raws represent k sequence, and the number of raws is a number of zeros n
    k=k(orderzero,:)+ phiamp;
    Q_0 = repmat(q_0,(2*Ntot+1),1)';%number of raws is n, columns 2n+1 and in columns are postions of zeros
    stack_q_0 = Q_0;   
    %%% allowing one minimum not to be taken into account:
    identity = diag(ones(1,N));
    % in total it was 2n+1 k path, but now we allow n-1 minima not to be
    % taken into account so stack_q0 and k in the end have dimentions nx (2n+1)*n
    noimport=0.95*min(min(Q_0));% to avoid mixing up very large and very small numbers
    for ii = 1 : N%index
        weigh = identity([1:(ii-1), (ii+1):end],:);
        stack_q_0 = [stack_q_0, [weigh*Q_0; noimport*ones(1,(2*Ntot+1))] ];
    end
    k = repmat(k,[1,N+1]);     
    %df_est = ( 2*k + stack_q_mi2.^4 ) ./ (2*stack_q_mi2.^2);
    df_est = ( lambda^3*Cs*stack_q_0.^4 - 2*k ) ./ (2*lambda*stack_q_0.^2);
    
    
    df_est_mean(1:(2*N+1)) = mean( df_est( :, 1:(2*N+1) ) );
    df_est_std(1:(2*N+1)) = std( df_est( :, 1:(2*N+1) ) );
    
    %exclude one zero
    df_est_mean = [df_est_mean, mean( df_est( 1:(end-1), (2*N+2):end ) )];
    df_est_std = [df_est_std, std( df_est( 1:(end-1), (2*N+2):end ) )];
    
    
    if signdf < 0
        
        df_est_std = df_est_std( find(df_est_mean < 0) );
        k = k( :,find(df_est_mean < 0) );
        df_est_mean = df_est_mean( find(df_est_mean < 0) );
        df_est_std_norm=df_est_std./abs(df_est_mean); % relative standard deviation
        [df_est_std_norm_min,df_est_std_norm_minpos]= min (df_est_std_norm); 
        minpos=df_est_std_norm_minpos;
     elseif signdf > 0
        df_est_std = df_est_std( find(df_est_mean > 0) ); % first we need to 
        %calculate std because if  we first calculate df_mean then it is going to be changed
        k = k( :,find(df_est_mean > 0) );
        df_est_mean = df_est_mean( find(df_est_mean > 0) );
        df_est_std_norm=df_est_std./abs(df_est_mean); 
        [df_est_std_norm_min,df_est_std_norm_minpos]=min (df_est_std_norm); 
        minpos=df_est_std_norm_minpos;
    else 
        % check overfocus
        df_est_std_ov = df_est_std( find(df_est_mean < 0) );
        k_ov = k( :,find(df_est_mean < 0) );
        df_est_mean_ov = df_est_mean( find(df_est_mean < 0) );
        df_est_std_norm_ov=df_est_std_ov./abs(df_est_mean_ov); % relative standard deviation
         % check underfocus  
        df_est_std_un = df_est_std( find(df_est_mean > 0) ); % first we need to 
        %calculate std because if  we first calculate df_mean then it is going to be changed
        k_un = k( :,find(df_est_mean > 0) );
        df_est_mean_un = df_est_mean( find(df_est_mean > 0) );
        df_est_std_norm_un=df_est_std_un./abs(df_est_mean_un); 
        if min(abs(df_est_std_norm_un)) < min(abs(df_est_std_norm_ov))
            df_est_mean=df_est_mean_un;
            df_est_std = df_est_std_un;
            k=k_un;
        [df_est_std_norm_min,df_est_std_norm_minpos]=min (df_est_std_norm_un);
        else
            df_est_mean=df_est_mean_ov;
            df_est_std = df_est_std_ov;
            k=k_ov;
        [df_est_std_norm_min,df_est_std_norm_minpos]=min (df_est_std_norm_ov);           
        end       
        minpos=df_est_std_norm_minpos;     
     end

    df = df_est_mean(minpos);
    std_df = df_est_std(minpos);
    k_out = k(:,minpos);
    
    if max(size(minpos))>1 || Cs==0 ;
        minpos = find(abs(df) == max(abs((df))));
        std_df = std_df(1);
        df = df(minpos);
        k_out = k(:,minpos);
        if Cs==0
            df = abs(df);
            k_out = k(:,1);
        end
    end
end
df_out = ( lambda^3*Cs*q_0.^4 - 2*k_out') ./ (2*lambda*q_0.^2);



