function [df, std_df, ellipticity, ast_angle, ast, std_ast] = estimate_df_ast(a, params)
% ESTIMATE_DF_AST determines the defocus, ellipticity of Thon rings,
%                 strength and orientation of astigmatism, together
%                 with corresponding uncertanties
%
% SYNOPSIS:
% [df, std_df, ellipticity, ast_angle, ast, std_ast] = estimate_df_ast(a,params)
%
% PARAMETERS:
%   a     : input image of (near-)amorphous sample that has at least one Thon
%           ring in the PSD
%   params: structure containing various physical and processing parameters
%
%
% OUTPUT:
%  df         : estimated defocus in [nm] (>0 for undefocus, <0 for
%               overfocus)
%  std_df     : uncertainty in the defocus estimation
%  ellipticity: estimated equivalent ellipticity of Thon ring
%  ast_angle  : estimated orientation of the astigmatism
%  ast        : estimated astigmatism in [nm] (>0 for undefocus, <0 for
%               overfocus)
%  std_ast    : uncertanty in the astigmatism estimation
%
%  NOTE: params structure contains the following fields:
%      a) physical   : bin,voltage,Cs,AmplitudeContrast
%      b) processing : DISPLAY, b_c2, b_f, parav_sem, iterNo, equi, prepro,
%                      use_df_mic, signdf, cs_corrrect, ast_strength,
%                      expectedratio, perfitlow, perfithigh, TRAs and TRAv

% (C) Copyright 2011
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
%  Milos Vulovic & Bernd Rieger

%a=inputa;
% inputpsd=squeeze(inputa);
% a=inputa;
%% ----------------SET DEFAULT VALUES FOR PROCESSING PARAMETERS------------
if ~isfield(params,'DISPLAY'); params.DISPLAY = 1;end          % display the results
if ~isfield(params,'perav_sem');params.perav_sem = 0;end  % perodogram averaging (yes=1, no=0);
if ~isfield(params,'iterNo');params.iterNo = 20;end % number of patches for periodogram averaging
if ~isfield(params,'equi');params.equi = 1;end  % outlier rejection according to equidistance in frequency sqaured space
if ~isfield(params,'prepro');params.prepro = 'adaptive';end %preprocessing options : 'adaptive', bilateral', 'adaptivespec', bilateralspec'
if (strcmp(params.prepro,'adaptivespec')|| strcmp(params.prepro,'bilateralspec'))&& (~isfield(params,'sigmas'))
    error('specify sigmas for "adaptivespec" and "bilateralspec" filter type. See details in prefilt.m') 
end
if strcmp(params.prepro,'bilateralspec')&& (~isfield(params,'spa_sig')||(~isfield(params,'tonal_sig')))
    error('specify spa_sig and tonal_sig for "bilateralspec" filter type. See details in prefilt.m') 
end
if ~isfield(params,'sigmas');params.sigmas = [2 1];end % sigmas of filter derivatives (used for 'adaptivespec' and 'bilateralspec')
if ~isfield(params,'spa_sig');params.spa_sig = [ ];end  % spatial sigmas for 'bilaterspec' filter
if ~isfield(params,'tonal_sig');params.tonal_sig = [ ];end % tonal sigma for 'bilaterspec' filter 
if ~isfield(params,'use_df_mic');params.use_df_mic = 0;end %the defocus value from mic. could be used as a help but must be specified!! 0='no', 1='yes'
if ~isfield(params,'signdf');params.signdf = 0;end % assumed it is not known (+1 if you know for sure it is undefocus, -1 for overfocus)
if ~isfield(params,'cs_correct');params.cs_correct = 1;end% corrects for the Cs influence in a case it is larger than 10 % of defocus influence: 0='no', 1='yes'
if ~isfield(params,'ast_strength');params.ast_strength = 'large';end %expected astigmatism 'large' or 'small', 'verysmall', 'none' or 'specify'
if strcmp(params.ast_strength,'verysmall')
    params.expectedratio = 1.002;
    params.perfitlow     = 0.1; 
    params.perfithigh    = 2.0;
elseif strcmp(params.ast_strength,'small')
    params.expectedratio = 1.02;
    params.perfitlow     = 0.1; 
    params.perfithigh    = 2.0;
elseif strcmp(params.ast_strength,'large')
    params.expectedratio = 1.2;
    params.perfitlow     = 0.1; % fitting up to 10% of the Cs influence on the aberration function 
    params.perfithigh    = 0.9; % fitting upto 90% of the Cs influence if astigmatism is strong
elseif strcmp(params.ast_strength,'specify')
    params.expectedratio = params.inratio;
    params.perfitlow     = 0.1; % fitting up to 10% of the Cs influence on the aberration function 
    params.perfithigh    = 0.9; % fitting upto 90% of the Cs influence if astifgmatism is strong
elseif strcmp(params.ast_strength,'none')
    params.cs_correct          = 0;% correct for the Cs influence in a
    params.TRAv                 = 0; % Thon ring averaging
    params.TRAs                 = 0; % Thon ring assesment (not possible without TRAv=1); 
    params.expectedratio = 1;
    params.perfitlow     = 0.1; % fitting up to 10% of the Cs influence on the aberration function 
    params.perfithigh    = 0.9; % fitting upto 90% of the Cs influence if astifgmatism is strong
else
    error('Unknown field for params.ast_strength');
end
if ~isfield(params,'TRAv');params.TRAv = 0;end  %Thon ring averaging (0 for no)  (1 for yes)
if ~isfield(params,'TRAs');params.TRAs = 0;end %Thon ring assesment (0 for no)  (1 for yes, requires TRAv=1)
imsize                  = size(squeeze(a));
szamin                  = min(imsize);
%--------------- Default rebinning of the image or the power spectrum.----- 
% For high mag (small pixel size) rebinning in real space is recommanded,
% for low mag --> rebinning in Fourier space. All these assume relatively
% high defocus (typical case for life-science). For very small defocus
% values the bellow default conditions (related to pixsize) could be changed accordingly.  
% For the optimal choise of binning and filtering parameters you can try
% the function "check_rings.m"
if ~isfield(params,'b_c2') || ~isfield(params,'b_f')
    if szamin==4096*2
        params.b_c2 = 2; % additional binning in the real space for faster calculation and SNR enhancement
        params.b_f  = 4; % additional binning in the frequency space for faster calculation and SNR enhancement
    elseif szamin < 4096*2 && szamin >= 4000
            if params.pixsize < 5e-11
            params.b_c2 = 4;
            params.b_f  = 2;
            elseif params.pixsize >= 5e-11 && params.pixsize < 0.1e-9
            params.b_c2 = 4;
            params.b_f  = 2;
            else
            params.b_c2 = 1;
            params.b_f  = 8;
            end         
    elseif szamin < 4000 && szamin >= 1024
            if params.pixsize < 1e-11
            params.b_c2 = 4;
            params.b_f  = 1;
            elseif params.pixsize >= 1e-11 && params.pixsize < 0.2e-9
            params.b_c2 = 2;
            params.b_f  = 2;
            else
            params.b_c2 = 1;
            params.b_f  = 4;
            end
    else
        params.b_c2 = 1;
        params.b_f  = 1;
    end
end
if ~isfield(params,'patchSz');params.patchSz = (szamin/params.b_c2)/2;end % patch size for periodogram averaging
if ~isfield(params,'AmplitudeContrast');params.AmplitudeContrast = 0.1;end %       <-- 10 % for carbon, 7% for cryo
    global Nsz2
 
%% ----------------------LOADING THE INPUT PARAMETERS-------------------
global DISPLAY
DISPLAY             = params.DISPLAY; % displays the results (yes=1, no=0)
displaypart         = 1; % part of the PSD to be displayed (default 100 %)
pixsize             = params.pixsize;% pixel size in specimen plane in [m]
bin                 = params.bin; % binning of acquired images
voltage             = params.voltage; % electron gun voltage in [V]
lambda              = wavlen(voltage);% wavelenght of the incident e-
Cs                  = params.Cs;% spherical aberration in [m]
b_c2                = params.b_c2;% additional spatial binning
b_f                 = params.b_f; % additional binning in Fourier space
sigmas              = params.sigmas; % sigmas of filter derivatives (used for 'adaptivespec' and 'bilateralspec')
spa_sig             = params.spa_sig; % spatial sigmas for 'bilaterspec' filter
tonal_sig           = params.tonal_sig;% tonal sigma for 'bilaterspec' filter 
perav_sem           = params.perav_sem;% periodogram averaging (yes=1,no=0)
iterNo              = params.iterNo; % # of patches for periodogram aver.
patchSz             = params.patchSz;% patch size for periodogram averaging
equi                = params.equi; % outlier rejection based on equidistant clustering
AmplitudeContrast   = params.AmplitudeContrast; %(0.1) carbon, (0.07)cryo
amplitudecontrast   = asin(AmplitudeContrast);
prepro              = params.prepro; % preprocessing PSD options :'adaptive', bilateral', 'adapativespec', 'bilateralspec'
use_df_mic          = params.use_df_mic;% the defocus value from mic. could be used as a help but must be specified!
signdf              = params.signdf; % +1 for undefocus, -1 for overfocus, 0 if not known.
if use_df_mic
    defocus = params.defocus;% if you want to use the defocus value from mic. as a help specify it here (in [nm])
    % (note that in most microscopes undefocus is <0, overfocus >0. Here the convention is oposite. 
    % Change the sign of the displayed defocus from the microscope!
end
cs_correct          = params.cs_correct;% correct for the Cs influence in a case it is 
%                                         larger than 10 % of defocus influence (yes=1, no=0);
ast_strength        = params.ast_strength; %expected astigmatism 'large','small', 'verysmall', 'none' or 'specify'
expectedratio       = params.expectedratio; %3 presets, large (1.2), small (1.02) or verysmall (1.002). It might be changed
%                                            in the second iteration to possibly get a more accurate estimation
perfitlow           = params.perfitlow; % fitting up to 100*perfitlow percent of the Cs influence
%                                         on the aberration function (default =0.1=10%)
perfithigh          = params.perfithigh;% take into account the rings up to 100*perfithigh 
%                                         percent of Cs influence in the 2nd pass
TRAv                = params.TRAv; % Thon ring averaging (yes=1, no=0)
TRAs                = params.TRAs; % Thon ring assesment (yes=1, no=0) (possible if TRAv=1);

%% -------------------------PREPROCESSING the PSD----------------------------
% as an additional help, the funtion check_rings.m could be used to check the
% best possible combination of binning and filtering parameters in order to 
% have the most prominent rings
minsz = min(size(squeeze(a)));
a = cut(a,minsz); % make the sqaured image  (it will be extended to rectangular case later)
% in order to allow the binning upto in total 16, it is desirable that image
% dimentions are divisable by 16
a = squeeze(cut(a,minsz - mod(minsz,16)));
imsize = size(a,1);%the size of the image
%a = a*hannwind(imsize); % apply the hann window to suppress edge effects
% ---spatial rebinning---
if b_c2 > 1 %rebin the image in the spatial domain first
    a = rebin(a,b_c2)/b_c2^2;
end
szpera = size(a,1);
% ---rebining due to the periodogram averaging---
fdrebin = 1;
if perav_sem
    [psd, fdrebin] = per_av(a, patchSz, iterNo);
else
% ---calculate the PSD---
    in = squeeze((a) - mean(a));
    clear a
    psd = abs(ft(in)).^2;
end
% ---additional rebinning in the Fourier domain---
if b_f > 1
    psd = rebin(psd,b_f)/b_f^2;
end
qtrue_pix = fdrebin*b_f/(pixsize*imsize); %pixel in the Fourier domain is fdrebin times larger, Nyquist should stay the same
if isfield(params,'qtrue_pix');qtrue_pix = params.qtrue_pix;end
lpsd = log(psd);

%The function check_rings.m could be used as a help to make the rings more
%pronounced and to chose better the following parameters
% sigmas   =[2 1];
% spa_sig  =[2 2];
% tonal_sig=3;
blog = prefilt(lpsd, prepro, sigmas, spa_sig, tonal_sig);%preprocess the PSD (filters bilateral and adaptive). 
N = size(blog,2);
radius_c = N/4;
cutpercent = 0.04; % cut the first 4% of the PSD
% ---predict the first zero from the microscope-provided defocus (if available)
if exist('defocus')
    disexpect1 = sqrt(1/abs(lambda*defocus*1e-9)); %this is when Cs=0
    if  disexpect1 <= cutpercent*qtrue_pix*N/2 % if the first zero is expected at quite low frequencies, cut less than cutpercent
        cutpercent = 0.5*(disexpect1/(qtrue_pix*N/2));
    end
    disexpect1 = sqrt(1/abs(lambda*defocus*1e-9));
    disexpect = disexpect1^2/qtrue_pix^2/(N/2);
end

%% --------------------Astigmatism is zero or user wants to neglect it: determine only defocus
if strcmp(ast_strength,'none') % (first 'if' : only defocus or both)
    Rfinal = 1; % the ellipticity is one
    sz = [N N];   
    blog = blog*(rr(N,N)>cutpercent*N);
  % ---angular averaging---
    FAST = 1; %if =1 uses radialmean.m function without weighting
   % if 0 Weighted angular averaging is performed by computing the weighted average inside rings with a Gaussian
   % profile to avoid problems arising from averaging too few data points at low spatial frequencies
    if ~FAST % weighted angular averaging    
        radiusstep = 2;
        radius = mk_cone(N, radiusstep);
        angularaverage = radav( blog, radius );
        angularaverage = resample(angularaverage,radiusstep);
    else %non-weighted angular averaging (default)
        angularaverage = radialmean(blog,[],1,1); %im2mat(radialmean(blog,[],2,1));
    end
  % ---maxima search---
    [ma_pos ma_sq1] = findmaxima_thon(angularaverage);
    sortall = ma_pos; missedzeros = [];
  % ---reject outliers---
    if equi && length(sortall)> 4
        if exist('defocus','var') % if you want to use defocus value from the microscope as a help
            [sortall, missedzeros] = reject_out(sortall, sz, disexpect);
        else
            [sortall, missedzeros] = reject_out(sortall, sz);
        end
    end
  % ---find defocus from zeros---
    zeroesPSD = (sortall)*qtrue_pix;
    Nzeros = length(zeroesPSD);
    phiamp = amplitudecontrast*gauss1D(zeroesPSD,0,radius_c*qtrue_pix,1)/pi;
    [df, std_df, k_out, df_out] = est_df( zeroesPSD', Cs, lambda, phiamp', missedzeros, signdf);
    defocus_est = df*1e9; std_df = std_df*1e9;  
    Astigm_est = 0; std_Astigm = 0;
    Rfinal_k = Rfinal;
    Nphi= 1; ysearch = -Nphi/2; phasecal = 0;  
  % ---predict the positions of the CTF zeros from the estimation--- 
    try
        [ qc, k, Rq] = predict_Thonshape(qtrue_pix, N, Cs, lambda, df, Astigm_est*1e-9, Nzeros);
    catch
        [ qc, k, Rq] = predict_Thonshape(qtrue_pix, N, Cs, lambda, df, Astigm_est*1e-9, Nzeros-1);
    end
    sortall= qc/qtrue_pix;  
else                                  % (first 'if' : only defocus or both)
    %% -----------------------CALCULATE BOTH ASTIGMATISM AND DEFOCUS------
    sigma = 1;
    minvalue = []; %the depth of the watershed segmentation (by default= 20%)
    sz = [N N];
    primerpol = dek2pol(blog, 'cubic'); % polar coordinates
    cutim = round(size(primerpol,1)*cutpercent);
    primerpol(0:cutim,:) = 0;
    ftin = ft(primerpol);
    %% --------------------------------template matching
  % ---making and matching the templates if needed, otherwise read from disk
    max_ratio = 2.2*(expectedratio - 1) + 1;
    max_ratio = max(1.0001,max_ratio);%in case expected ratio is extremely small
    A1 = 1:(max_ratio - 1)/50:max_ratio; %makes 50 templates with ellipticieties between 1 and max_ratio
    template_settings % load the directory where (if available) templates are stored

    try
        dR = A1(2) - A1(1); %the step
        mR = A1(end); %maximal ellipticity
        if DISPLAY;fprintf('Trying to use template %s ', ...
                ['img_' num2str(N) '_dR' num2str(dR) 'Rmax' num2str(mR) '.ics']);end
        templates = readim([ templateDIR 'img_' num2str(N) '_dR' num2str(dR) 'Rmax' num2str(mR) '.ics']);
        if DISPLAY;fprintf('  -- OK\n');end
    catch
        if DISPLAY;fprintf('Template not found -- generating new template.\n');end
        templates = gen_templates(A1,N,1);
    end
    out = newim([size(primerpol) length(A1)],'sfloat'); %preallocation
    jj = 0;
    for ii = A1
        tmp = real(ift(ftin*squeeze(templates(:,:,jj)))); % template matching
        out(:,:,jj) = tmp; % 3D parameter space image
        jj = jj+1;
    end
    clear tmp ftin jj
  % ---Extend the parameter space in order to detect maxima for zero astigmatism or pi/4 ast. orientation
    offst = 3;
    outmirrR = mirror(out(:,:,1:offst), 'z-axis');
    outmirrsh = shift(outmirrR,[0, size(primerpol,2)/2, 0]);
    outextR = cat(3, outmirrsh, out); %extending in R<1 direction
    outmirrF = mirror(outextR(:,end-1:-1:end-offst,:), 'y-axis');
    outextF = cat(2, outmirrF, outextR); %extending in angle direction
    if ~isempty(minvalue)
        minvalue = (max(out) - min(out))*minvalue;
    end
    [ma, saliency] = findmaxima_thon(outextF,minvalue,'joint'); %search for the maxima
    if isempty(ma)
        warning('Could not find any good maxima in the 3D Parameter space, try with separate..');
        [ma, saliency] = findmaxima_thon(outextF,minvalue,'separate');
    end
    %ma(ma(:,2)<offst | ma(:,3)<offst, : )=[];
    %saliency(ma(:,2)<offst | ma(:,3)<offst, : )=[]; %exclude the found maxima that are outside the original parameter space
    uslov = 0;
    if ismember(median(ma(:,3)), [size(outextF,3)-offst:size(outextF,3)])
       uslov=1;  
    end
    ma(:,2) = ma(:,2) - offst;
    ma(:,3) = ma(:,3) - offst;
    % in the case astigmatism could not be detected, we proceed with defocus only
    if isempty(ma) || strcmp(ast_strength,'none') || uslov %(2nd 'if': we want to estimate both, but astigm is not detectable') 
       fprintf('Astigmatism could not be calculated. Estimating defocus only...')
        [ma, saliency] = findmaxima_thon(squeeze(mean(out, [],[2,3])), minvalue, 'separate'); 
        Rfinal = 1;
        sz = [N N];    
        sortall = ma; missedzeros = [];
      % ---reject outliers---
        if equi && length(sortall)> 4
         equisem=1;
         outlrs=[ ];
             if exist('defocus','var')
                 [sortall, missedzeros] = reject_out(sortall, sz, disexpect);
             else
                 [sortall, missedzeros] = reject_out(sortall, sz);
             end
        end
      % ---find defocus from zeros---    
        zeroesPSD = (sortall)*qtrue_pix;
        Nzeros = length(zeroesPSD);
        phiamp = amplitudecontrast*gauss1D(zeroesPSD,0,radius_c*qtrue_pix,1)/pi;
        [df, std_df, k_out, df_out] = est_df( zeroesPSD', Cs, lambda, phiamp', missedzeros, signdf);
        defocus_est = df*1e9; std_df = std_df*1e9;  
        Astigm_est = 0; std_Astigm = 0;
        Rfinal_k = Rfinal;
        Nphi= 1; ysearch = -Nphi/2; phasecal = 0;  
      % ---predict the positions of the CTF zeros from the estimation 
        try
            [ qc, k, Rq] = predict_Thonshape(qtrue_pix, N, Cs, lambda, df, Astigm_est*1e-9, Nzeros);
        catch
            [ qc, k, Rq] = predict_Thonshape(qtrue_pix, N, Cs, lambda, df, Astigm_est*1e-9, Nzeros-1);
        end   
        sortall= qc/qtrue_pix; 
    else                     %(2nd 'if': we want to estimate both, and both are detectable') 
        %     
        % stepedges = 1;
        % edges = min(ma(:,2)):stepedges:max(ma(:,2));
        % [index,BIN] = histc(ma(:,2),edges );
        % [modefind, valuemode]=mode(BIN);
        % if modefind==0 || valuemode < 2
        %     ysearch = median(ma(:,2));
        % else
        %     edges([modefind, modefind+1]);
        %     % hist(ma(:,2),edges+.5);
        %     ysearch = (edges(modefind) + edges(modefind+1))/2;
        % end
        if ~isempty(ma) %&& max(valysearch)<length(kra)-5
          ysearch = median(ma(:,2));
        end
        ysearchext = ysearch + offst;
        outint = squeeze(outextF(:,round(ysearchext),:));
        %linear interpolation of slices
        if  round(ysearchext) ~= (ysearchext)
            outpre = squeeze(outextF(:,floor(ysearchext),:));
            outpos = squeeze(outextF(:,ceil(ysearchext),:));
            coef = (outpos-outpre)/(ceil(ysearchext) - floor(ysearchext));
            outint = outpre+coef*(ysearchext - floor(ysearchext));
        end
        clear out
        [m, saliency] = findmaxima_thon(outint,minvalue,'joint'); %search in 2D space  
            if isempty(m)
                warning('Could not find any good maxima in 2D parameter space. Try with separate...');
                [m, saliency] = findmaxima_thon(outint,minvalue,'separate');
            end
        m(:,2) = m(:,2) - offst;
        try
            mx(:,1:2) = repmat(m(m(:,1)>cutim),[1 2]); %only the ones larger than cutim
            mxlen = length(m(:,1))-length(mx(:,1));
            mx(:,2) = m(mxlen+1:end,2);
            m = mx;
        catch
            fprintf(' - not cutting out the very low freq rings\n');
        end
        if DISPLAY; fprintf(' -- Found %d zero crossing in the PSD --\n',size(m,1));end
        if isempty(m)
            warning('Could not find any good maxima in the Radon Transform.');
            defocus_est = []; Astigm_est = []; std_df = []; phasecal = []; Rfinal = [];
            ellipticity = Rfinal; ast_angle = phasecal; df = defocus_est;
            ast = Astigm_est; std_ast = [];
            return;
        end
        clear outint
        m(m(:,2)+1<1,2) = 1;    
        rnum = interp1(A1,m(:,2)+1);
        xovi = 1:length(rnum);
        yovi = rnum;
        p = polyfit(xovi, yovi',1);
        if abs(p(1)) < 0.0000002 
            rnum = rnum-p(2)+1;
        end
      % ---correction for the mean value of rcurve in template and detected frequency
        r_erroresp = zeros(1,length(rnum));
        t = linspace(0, pi, 100);
        jj = 0;
        for incr = squeeze(rnum')
            r_ellpse2pol = 1./sqrt(1+ (incr^2-1)*sin(t).^2);
            resp = sqrt(2./(incr^2+1));
            rest = mean(r_ellpse2pol);
            r_erroresp(:, jj+1) = (rest-resp)/rest;
            jj = jj+1;
        end
        r_errorcorr = r_erroresp;
        out2 = [((m(:,1) - radius_c).*(1 - r_errorcorr') + radius_c), repmat(ysearch,[size(m,1),1]), rnum];
        out2orig = out2;

      % ---find the angles of major axes of ellipse
        Nphi = size(primerpol, 2);
        phasecal = (3*Nphi/2-ysearch)/Nphi*180; %estimated phase of the sine-like curve
        if ysearch < Nphi/2
            phasecal = (Nphi/2-ysearch)/Nphi*180;
        end
        phaser = phasecal*pi/180; % estimated phase in radians

     %%  ----------------- first initial (rough) guess for defocus
        if strcmp(ast_strength,'small') || strcmp(ast_strength,'verysmall') 
            sz = [N N];   
            blog = blog*(rr(N,N)>cutpercent*N);
          % ---non-weighted angular averaging (default)
            angularaverage = radialmean(blog,[],1,1); %im2mat(radialmean(blog,[],2,1));
          % ---maxima search 
            [ma_pos ma_sq1] = findmaxima_thon(angularaverage);
            sortall = ma_pos; missedzeros = [];
        else
           sortall = out2(:,1); missedzeros = [];
           sortalln = sortall; 
           %try to estimate defocus only from the first half of the spectrum
           if length(sortall(sortall(:,1)<N/4))>0
                sortalln = sortall(sortall(:,1)<N/4);
           end  
           sortall=sortalln;
        end  
      % ---reject outliers
        if equi && length(sortall)> 4
            if exist('defocus','var') % if you want to use defocus value from the microscope as a help
                [sortall, missedzeros] = reject_out(sortall, sz, disexpect);
            else
                [sortall, missedzeros] = reject_out(sortall, sz);
            end
        end
      % ---find defocus from zeros---
        zeroesPSD = (sortall)*qtrue_pix;
        Nzeros = length(zeroesPSD);
        phiamp = amplitudecontrast*gauss1D(zeroesPSD,0,radius_c*qtrue_pix,1)/pi;
        [df, std_df, k_out, df_out] = est_df( zeroesPSD', Cs, lambda, phiamp', missedzeros, signdf);    
        defocus_est_init = df*1e9; std_df_init = std_df*1e9;
        k_outinic = k_out;
        defocus_better = defocus_est_init;

      % if the error is 10% of the estimated defocus use the one that is given by microscope?
        if (std_df_init > 0.1*defocus_est_init) && exist('defocus','var')
            defocus_better = defocus;
            std_df_init = 0;
        end

        if DISPLAY
            fprintf('\n initial defocus estimate [nm]\n');
            fprintf(' ---------------------------------------------------------------------------------\n');
            fprintf('  %4.0f +- %2.0f\t\n', defocus_better, std_df_init);
        end
      % ---Fit only up to the influence of 10% of Cs term 
      % this 10% corresponds to the error of 25% in equidistance in q2 space
        qfit = sqrt(2*perfitlow*abs(defocus_better*1e-9)/(lambda^2*Cs)); %requires an estimate of defocus
        qmaxfit = sqrt(2*perfithigh*abs(defocus_better*1e-9)/(lambda^2*Cs));
        qmax = qtrue_pix*max(out2orig(:,1)); %maximal detected frequency
        qdet = out2(:,1)*qtrue_pix;
        clear out2orig_doCs
        out2orig_doCs(:,:) = out2orig([1:length(qdet<qmaxfit)]',:);
        global Nsz2
        Nsz2 = size(primerpol,1)/2;

     %% ----- -Estimate defocus and astigmatism without Cs correction
        if (qmax < qfit) || (cs_correct==0) % without Cs correction     (3rd 'if': perform Cs correction or not) 
          % ---find the ellipticity---
            [Rfinal, std_Rfinal] = find_ellipticity(out2, qtrue_pix);
          % ---for small astigmatism defocus can be found independently of ellipticity 
            if Rfinal < 1.02   
                sz = [N N];   
                blog = blog*(rr(N,N)>cutpercent*N);
              % ---non-weighted angular averaging (default)
                angularaverage = radialmean(blog,[],1,1); %im2mat(radialmean(blog,[],2,1));
              %---maxima search ---
                [ma_pos ma_sq1] = findmaxima_thon(angularaverage);
                %[ma_pos, saliency] = findmaxima_thon(squeeze(mean(out, [],[2,3])), minvalue, 'separate'); 
                sortall = ma_pos; missedzeros = [];
            else
                sortall = out2(:,1); % use maxima from parameter space
            end  
          % ---reject outliers---
            if equi && length(sortall)> 4
                if exist('defocus','var') % if you want to use defocus value from the microscope as a help
                    [sortall, missedzeros] = reject_out(sortall, sz, disexpect);
                else
                    [sortall, missedzeros] = reject_out(sortall, sz);
                end
            end
          % ---find defocus from zeros---
            zeroesPSD = (sortall)*qtrue_pix;
            Nzeros = length(zeroesPSD);
            phiamp = amplitudecontrast*gauss1D(0,zeroesPSD,radius_c*qtrue_pix,1)/pi;      
            [df, std_df, k_out, df_out] = est_df( zeroesPSD', Cs, lambda, phiamp', missedzeros, signdf);
            defocus_est = df*1e9;
            std_df = std_df*1e9;  
            %     if std_df_init/defocus_est_init < std_df/defocus_est
            %         defocus_est=defocus_est_init;
            %         std_df=std_df_init;
            %     end
            Astigm_est = defocus_est*abs((Rfinal^2-1)/(Rfinal^2+1)); %Rfinal is computed at Nsz2 (half the freq range)
            std_Astigm = sqrt( ((Rfinal^2-1)/(Rfinal^2+1)*std_df)^2 + (4*defocus_est*Rfinal/(Rfinal^2+1)^2 *std_Rfinal)^2 );
            Rfinal_k = Rfinal;   
          % ---predict the positions of the CTF zeros from the estimation
            try
                [ qc, k, Rq] = predict_Thonshape(qtrue_pix, N, Cs, lambda, df, Astigm_est*1e-9, Nzeros);
            catch
                [ qc, k, Rq] = predict_Thonshape(qtrue_pix, N, Cs, lambda, df, Astigm_est*1e-9, Nzeros-1);
            end
            sortall= qc/qtrue_pix;   

     %% ----estimation with refinement!!!!! 2 step approach to correct for the Cs influence
        elseif  (qmax >= qfit) && (cs_correct==1)  %  % (3rd 'if': perform the Cs correction or not) 
            rejectedzeros = find((qdet>qfit) | (qdet==qfit));
            out2(rejectedzeros,:) = [];
            [Rfinal, std_Rfinal] = find_ellipticity(out2,qtrue_pix);
            sortall = out2(:,1); missedzeros = [];
          % ---reject outliers---
            if ~isempty(sortall)
                if equi && length(out2(:,1))> 4
                    if exist('defocus','var')
                        [sortall, missedzeros] = reject_out(sortall, sz, disexpect);
                    else
                        [sortall, missedzeros] = reject_out(sortall, sz);
                    end
                end
            else
                error('could not find any CTF minima')
            end
          % ---find defocus from zeros---
            zeroesPSD = (sortall)*qtrue_pix;
            phiamp = amplitudecontrast*gauss1D(0,zeroesPSD,radius_c*qtrue_pix,1)/pi;
            [df, std_df, k_out, df_out] = est_df( zeroesPSD', Cs, lambda, phiamp', missedzeros, signdf);
            defocus_est = df*1e9; std_df = std_df*1e9;
            Astigm_est = defocus_est*(Rfinal^2-1)/(Rfinal^2+1);
            std_Astigm = sqrt( ((Rfinal^2-1)/(Rfinal^2+1)*std_df)^2 + (4*defocus_est*Rfinal/(Rfinal^2+1)^2 *std_Rfinal)^2 );

            if DISPLAY
                fprintf('\nEstimation without Cs correction\n')
                fprintf('   ratio       defocus [nm]      astigmatism [nm]\n');
                fprintf(' ---------------------------------------------------------------------------------\n');
                fprintf('%9.4f\t  %6.0f +- %2.0f\t\t %6.1f +- %3.1f\n', Rfinal,  defocus_est, std_df, Astigm_est, std_Astigm);
            end

          % ---taking into account all the zeros to find  Req---
            Q = out2orig_doCs(:,3); % ellipticities
            qc = out2orig_doCs(:,1)*qtrue_pix; % central frequency positions of the rings
            qs = sqrt(qc.^2.*(Q.^2+1)./(2*Q.^2)); % short axes
            ql = sqrt(qc.^2.*(Q.^2+1)/2); % long axes
            dfl_est = (defocus_est-Astigm_est)*1e-9;
            dfs_est = (defocus_est+Astigm_est)*1e-9;
            Qeq = Rfinal.*Q.*sqrt((2*dfl_est-Cs*lambda^2.*ql.^2)./(2*dfs_est-Cs*lambda^2.*qs.^2)); % normalized ellipticities using the initial estimate (eq C.12)
            Requi = Rfinal;
            out2Cs = [out2orig_doCs(:,1)  Requi*ones(length(Qeq),1) Qeq];
            out2old = [out2orig_doCs(:,1) out2orig_doCs(:,1) Q];
            [Rfinal, std_Rfinal] = find_ellipticity(out2old, qtrue_pix, out2Cs);
            Requi = Rfinal;
            sortall = out2orig_doCs(:,1);
            missedzeros = [];
            zeroesPSD = (sortall)*qtrue_pix;
            Nzeros = length(zeroesPSD);

          % ---find defocus---
            phiamp = amplitudecontrast*gauss1D(0,zeroesPSD,radius_c*qtrue_pix,1)/pi;
            [df, std_dffull, k_outfull, df_out] = est_df( zeroesPSD', Cs, lambda, phiamp', missedzeros, signdf);
            defocus_estfull = df*1e9; std_dffull = std_dffull*1e9;
            equisample = 1;
            if std_dffull/abs(defocus_estfull) < std_df/abs(defocus_est)
                defocus_est = defocus_estfull;
                std_df = std_dffull;
            end
            k_out = k_outfull;
            Astigm_est = defocus_est*(Requi^2-1)/(Requi^2+1);
            std_Astigm = sqrt( ((Requi^2-1)/(Requi^2+1)*std_df)^2 + (4*defocus_est*Requi/(Requi^2+1)^2 *std_Rfinal)^2 );

          % ---predict the position of the CTF zeros from the estimation---    
            try
                [ qc, k, Rq] = predict_Thonshape(qtrue_pix, N, Cs, lambda, df, Astigm_est*1e-9, Nzeros);
            catch
                [ qc, k, Rq] = predict_Thonshape(qtrue_pix, N, Cs, lambda, df, Astigm_est*1e-9, Nzeros-1);
            end  
            sortall= qc/qtrue_pix;
            R = Rq;
            if DISPLAY
                figure(74)
                plot(abs(k), R);
                title ('Ellpticity of the rings')
                xlabel('magnitude of k value of the ring')
                ylabel('ellipticity')
            end
            Rfinal_k = R;
        end % (3rd 'if': do the Cs correction or not)
    end %(2nd 'if': we want to estimate both, but astigm is not detectable') 
end  % (first 'if' : only defocus or both)


%% -------------plot corresponding ellipses and display the result
if DISPLAY
    radius_short = sortall.*sqrt((Rfinal_k.^2+1)./(2*Rfinal_k.^2));
    radius_long = radius_short.*Rfinal_k;
    numbzeros = length((sortall));
    angleell = ysearch/Nphi*pi + pi/2;
    orientradlong = repmat(angleell, [1 numbzeros]);
    dipshow(lpsd, 'percentile')
    ellipse(radius_long, radius_short, orientradlong , N/2, N/2);% ['r', 'b']);
end

if DISPLAY
    fprintf('\n   ratio   angle [deg]   defocus [nm]     astigmatism [nm]\n');
    fprintf(' ---------------------------------------------------------------------------------\n');
    fprintf('%9.4f\t %9.0f\t %6.0f +- %2.0f\t %6.1f +- %3.1f\n', Rfinal, phasecal, defocus_est, std_df, Astigm_est, std_Astigm);
end
%% -------------------------- output of the function
ellipticity = Rfinal;
ast_angle = phasecal;
df = defocus_est;
ast = Astigm_est; std_ast = std_Astigm;

%% --------------Thon ring averaging TRAv and assesment TRAs
if TRAv && DISPLAY
    imanalyze = lpsd;
    sz = size(imanalyze);
    radiusRange = [0 sz(1)/2-1];
    radiusStep = 1;
    defocusFEI = -defocus_est*1e-9;
    astigFEI =Astigm_est*1e-9;
    astigmatismAngle = phasecal*pi/180;
    
    amplitudeContrastFraction = amplitudecontrast/pi;
    CTFmodel = [lambda Cs defocusFEI astigFEI astigmatismAngle amplitudeContrastFraction];% complete model TRA
    CTFmodel1 = [lambda Cs defocusFEI 0 0 amplitudeContrastFraction]; %circular averaging
    CTFmodel2 = [lambda 0 defocusFEI astigFEI astigmatismAngle amplitudeContrastFraction]; % elliptical averaging
    %physicalBinningStep = 2 * 4328734.4858156024;
    physicalBinningStep = qtrue_pix;
    if b_c2==1 && b_f==1 && perav_sem==0
        sigmaRange = [2 1]; % gaussian blurring of the spectrum to suppress the noise
    elseif  (b_c2>1 && b_f>1) && perav_sem==1
        sigmaRange = [0.5 0.5];
    else
        sigmaRange = [1.2 0.9];
    end
    %bla = radialmean(double(imanalyze));
    %blapl = bla(0:sz(1)/2-1);
    %xax = radiusRange(1):radiusRange(end);
    
    if ~exist(['ThonRingAverage_improved.' mexext],'file')
        try
            mex  ThonRingAverage_improved.c
        catch 
            error('Cannot perform thon ring averaging, no compiler installed and no mex file for this architecture provided.');
        end
    end
    out = ThonRingAverage_improved(double(imanalyze), radiusRange,radiusStep, CTFmodel, physicalBinningStep, sigmaRange);
    out1 = ThonRingAverage_improved(double(imanalyze), radiusRange,radiusStep, CTFmodel1, physicalBinningStep, sigmaRange);
    out2 = ThonRingAverage_improved(double(imanalyze), radiusRange,radiusStep, CTFmodel2, physicalBinningStep, sigmaRange);
    if (qmax < qfit) || (cs_correct==0)
        out = out2; % TRA is elliptical averaging if Cs correction is not performed
    end
    
    figure(15)
    xax = 0:displaypart*(radiusRange(end)+1)-1;
    plot(xax*qtrue_pix/1e9, out1(1:displaypart*length(out1)),'b', xax*qtrue_pix/1e9, out2(1:displaypart*length(out2)), 'k', xax*qtrue_pix/1e9,out(1:displaypart*length(out)), 'r','Linewidth',2) %,xax, blapl, 'g');
    legend('Circular averaging', 'Elliptical averaging','Thon ring averaging') %, 'angular')
    xlabel('Frequency [nm^{-1}]')
    title(['1D averaged spectrum for defocus of ', num2str(round(defocus_est)), 'nm and astigmatism of ' ,  num2str(round(Astigm_est)), ' nm'])
    ylabel('log(PSD) [a.u.]')
    %ylim([9 17])
    xlim([0.7 displaypart*xax(end)*qtrue_pix/1e9])
    hold on
    
    % --------------------------Thon ring assesment TRAs
    if TRAs && std_df~=0
        masknf = (rr(size(imanalyze))>size(imanalyze,1)/2);
        selectlpsd = imanalyze*masknf;
        nf = exp(double(median(selectlpsd,masknf,[1 2])));
        %Simulate image with given parameters
        %%% CTF based on estimated parameters
        q_new = 0:qtrue_pix:qtrue_pix*(size(out,1)-1);
        %q_new=(radiusRange(1):radiusRange(end))*qtrue_pix;
        %ampcon = amplitudecontrast * linspace(1, 0, size(imanalyze,1));
        CTFest = abs(sin(2*pi*(0.25*Cs*lambda^3*q_new.^4-0.5*defocus_est*1e-9*lambda*q_new.^2 ) - amplitudecontrast)); %.^2;
        % plot(q_new/1e9, 13-CTFest)
        predicted_max = findmaxima_thon(CTFest);
        predicted_min = findmaxima_thon(-CTFest);
        val_max = interp1( q_new/qtrue_pix,exp(out),predicted_max);
        val_max_mean = (val_max(1:end-1)+val_max(2:end))/2;
        val_min = interp1( q_new/qtrue_pix,exp(out),predicted_min);
        val_det = interp1( q_new/qtrue_pix,exp(out),zeroesPSD'/(qtrue_pix));
        %qrange_ass = q_new(find((val_max_mean-val_min(1:length(val_max_mean)) > nf)));
        qcut = find(((val_max_mean-val_min(1:length(val_max_mean)) > nf)==0))-1;
        if (length(qcut)>1) && (qcut(1)==0)
            qcut(1)=[];
        end
        
        hold on
        plot(predicted_min*qtrue_pix/1e9, log(val_min), 'bo', 'MarkerSize', 16)
        hold on
        plot(predicted_max*qtrue_pix/1e9, log(val_max), 'mx', 'MarkerSize', 16)
        hold on
        plot(zeroesPSD'/1e9, log(val_det) ,'r+', 'MarkerSize', 16)
        plot(predicted_max*qtrue_pix/1e9, log(nf)*ones(1,length(predicted_max)), 'g-')
        ylim([0.8*log(nf) 1.1*max(log(val_max));])
        legend('Circular averaging', 'Elliptical averaging','Thon ring averaging','predicted minima', 'predicted maxima', 'detected minima', 'noise floor')
        %xlim([0.7 displaypart*xax(end)*qtrue_pix/1e9])
        scale = max(log(val_max))+log(nf);
        if ~isempty(qcut)
            qranicno = predicted_min(qcut(1))*qtrue_pix;
            %numzero = qcut(1);
            y = scale*ones(1,length(qranicno));
            stem(qranicno/1e9,y,'r-')
        else
            error('Could not assess the Thon rings')
        end
        
       %elliptical
        val_max_ell = interp1( q_new/qtrue_pix,exp(out2),predicted_max);
        val_max_mean_ell = (val_max_ell(1:end-1)+val_max_ell(2:end))/2;
        val_min_ell = interp1( q_new/qtrue_pix,exp(out2),predicted_min);
        qcut = find(((val_max_mean_ell-val_min_ell(1:length(val_max_mean_ell)) > nf)==0))-1;
        if (length(qcut)>1) && (qcut(1)==0)
            qcut(1)=[];
        end
        qranicno = predicted_min(qcut(1))*qtrue_pix;
        %numzero_ell = qcut(1);
        stem(qranicno/1e9,y,'k-')
        hold on
        %circular
        val_max_circ = interp1( q_new/qtrue_pix,exp(out1),predicted_max);
        val_max_mean_circ = (val_max_circ(1:end-1)+val_max_circ(2:end))/2;
        val_min_circ = interp1( q_new/qtrue_pix,exp(out1),predicted_min);
        %qrange_ass_circ = q_new(find((val_max_mean_circ-val_min_circ(1:length(val_max_mean_circ)) > nf)));
        %qassesed = max(qrange_ass_circ);
        %qinrange_circ = length(qrange_ass_circ);
        qcut = find(((val_max_mean_circ-val_min_circ(1:length(val_max_mean_circ)) > nf)==0))-1;
        if (length(qcut)>1) && (qcut(1)==0)
            qcut(1) = [];
        end
        qranicno = predicted_min(qcut(1))*qtrue_pix;
        %numzero_circ = qcut(1);
        stem(qranicno/1e9,y,'b-')
        hold off
    end
end
