%  --This is an example that demonstrates the defocus and astigmatism estimation with
%    Cs correction, for underfocus (df>0, A1>0), high magnification
%    and relatively large astigmatism. 

%% ----input parameters for defocus and astigmatism estimation
%  They must be provided as an input for the function estimate_df_ast.m
%  The parameters are described in a structure file struct and they are 
%  organized in 2 groups. 
%  The first group consists of the parameters that must be 
%  provided (obligatory!) and they describe imaging conditions. The
%  test unbinned image is acquired at a magnification of 380k on a 
%  microscope operated at 300kV voltage and equipped with 4k camera. 
%  Spherical aberration of objective lens is 2.7 mm.  
%  The 2nd group includes the processing parameters (optional). Their default values 
%  are specified in estimate_df_ast.m. However, if you want to change them to 
%  suit better your acquisition (see section 3 in def_ast_estimator_manual
%  for some recommendations), you can uncomment the line (from 54-85 in
%  this file) and change any value. If you want to use only default values
%  please comment all the lines from 54-85). In this example we changed 
%  only the default setting of showing Thon ring averaging and assesment 
%  which in this case shows clearly the difference between TRA, elliptical,
%  and circular averaging since Cs correction was performed, and astigmatism 
%  is large
%  NOTE that the convention for defocus sign in this program is opposite from the one used in most microscopes 
clear all
close all

% ------reading the image
imagename = 'Image4example3';
a = readim(imagename);

%% -----obligatory input (acquisition conditions)
% --1a) pixel size
% you can either specify the (post)magnification and physical pixel size or
% directly specify the pixel size in the specimen plane:

% 1a1)mag and postmag and physical pixel size
magmic          = 380000; %                      <--specify magnification
CLfac           = 1.875616608960196; %           <--specify post-magnification factor
%                                                   between plane of the film
%                                                   plate and your camera
ph_pixsize      = 15e-6; %                       <--specify CCD physical pixel size
params.bin      = 1; %                           <-- specify binning of acquired images
params.pixsize  = ph_pixsize*params.bin/(magmic*CLfac);
% 1a2) or specify pixel size in the image plane directly
params.pixsize  = params.pixsize; %              <--or specify pixel size directly

% --1b) microscope parameters
params.voltage  = 300e3;%                        <--specify voltage
params.Cs       = 2.7e-3;%                       <--specify spherical aberration

%% ----- Processing parameters (optional) ----------

%params.prepro       = 'adaptive'; %preprocessing options : 'adaptive',
%                                   'bilateral', 'adaptivespec', 'bilateralspec'.
%                                    Default:'adaptive'
%params.b_c2         = 4;   % real space binning (options: 1,2,4,8).
%                             Default: depends on the pixel size
%params.b_f          = 1;   % binning in Fourier domain (options:1,2,4,8)
%                              Default: depends on the pixel size
%params.sigmas       = [2 1]; % sigmas of filter derivatives (used for
%                             'adaptivespec' and 'bilateralspec'
%params.spa_sig      = 2; % spatial sigmas for 'bilaterspec' filter
%params.tonal_sig    = 10;     % tonal sigma for 'bilaterspec' filter                                                  
%params.signdf       = 1;   % use the knowledge whether it is
%                             underfocus (= +1) or overfocus (=
%                             -1).Default: unknown(=0)
%params.use_df_mic   = 0;   % use the value from the microscope
%                             as a help (yes=1, no=0); If =1,
%params.defocus = -1000;    % <-- put here the value from the
%                             microscope (e.g. -1000 means 100nm undefocus)
%params.perav_sem    = 1;   % perodogram averaging (yes=1, no=0); Default: 0.
%params.iterNo       = 20;  % number of patches for periodogram averaging
%params.patchSz      = 512; % Patch size for periodogram averaging 
%                             Default: half of the size of the PSD
%params.DISPLAY      = 1;   % display the results (1for yes, 0 for no). Default:1.
%params.equi         = 1;   %outlier rejection according to equidistance in q2.
%                            (yes=1, no=0); Default:(= 1).   
%params.cs_correct   = 1;   %correct for Cs influence where
%                            necessary (yes=1, no=0). Default (= 1)
%params.ast_strength = 'none';% expected astigmatism: %'large', 'small' Default:'large'
%params.expectedratio = 1.2; %expected ellipticity. (for possible refinement)
params.TRAv = 1; %Thon ring averaging: (yes=1, no=0).  Default (= 0)
params.TRAs = 1; %Thon ring assesment: (yes=1, no=0).  Default (= 0)
%params.AmplitudeContrast = 0.1; %   <-- 0.1 for carbon, 0.07 for cryo

%% ----ESTIMATE DEFOCUS AND ASTIGMATISM AND CORRESPONING UNCERTINTIES!
[df, std_df, ellipticity, angle, ast, std_ast] = estimate_df_ast(a,params);


