function  [Rfinal, std_Rfinal] = find_ellipticity(out2,qtrue_pix, out2corr)
% FIND_ELLIPTICITY calculates the apparent ellipticity common for all the
%                  rings by fitting a model function through the template
%                  ellipticities
%
% SYNOPSIS:
% [Rfinal, std_Rfinal] = find_ellipticity(out2,qtrue_pix, out2corr)
%
% PARAMETERS:
%     out2: matrix with first collumn containing the positions of the
%           found maxima from template matching and with the third collumn
%           conatining the apparent ellipticities for that positions
%qtrue_pix: pixel size in the Fourier domain
% out2corr: in case of Cs correction, this is the matrix that should be
%           used for fitting as its values are corrected for the CS
%           influence
%
% OUTPUT:
%      Rfinal: estimated equivalent ellipticity
%  std_Rfinal: uncertainty in the ellipticity estimation

% (C) Copyright 2011
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
%  Milos Vulovic & Bernd Rieger

global DISPLAY
% global inratio
% global Nsz2
if nargin==3
    Cscorrected=1;
    xfit=[0; out2corr(:,1)];
    yfit=[1; out2corr(:,3)]; % We are sure that it should pass through (x,y)=[0,1];
else
    xfit=[0; out2(:,1)];
    yfit=[1; out2(:,3)]; % We are sure that it should pass through (x,y)=[0,1];
    Cscorrected=0;
end

beta=1;
opt = statset;
if length(xfit)> 5
    opt.Robust = 'on'; %robust fitting removes outliers
else
    opt.Robust = 'off';
end
opt.TolFun = 1e-10;
opt.TolX = 1e-10;
[Rfinal_model, mod_Res, mod_Jacob, mod_Covariance, mod_MSE] = nlinfit(xfit,yfit,@modelfunc,beta,opt);
ci1s = nlparci(Rfinal_model,mod_Res,'covar',mod_Covariance,'alpha',.325); %67.5% of data is present
ci3s = nlparci(Rfinal_model,mod_Res,'covar',mod_Covariance,'alpha',.001); %99.9% of data is present
std_Rfinal_model = abs(Rfinal_model - ci1s(2));
std_Rfinal = std_Rfinal_model;
Rfinal = Rfinal_model;

if DISPLAY
    %fprintf('std_Rfinal %2.4f+-%2.5f\n', Rfinal_model,std_Rfinal)
    xp = linspace(0,max(xfit),100);
    yp = modelfunc(Rfinal, xp);
    figure;hold on
    leg_count=1;
    if ~isempty(out2)
        h(leg_count) = plot(out2(:,1)*qtrue_pix/1e9,out2(:,3), 'bx','Linewidth',1);
        l{leg_count} = 'zero-cross';leg_count=leg_count+1;
    else
        error('Could not find any minima')
    end
    
    if Cscorrected
        h(leg_count) = plot(out2corr(:,1)*qtrue_pix/1e9,out2corr(:,3), 'rx','Linewidth',1);
        l{leg_count} = 'corrected zero-cross';leg_count=leg_count+1;
    end
    
    h(leg_count) = plot(xp*qtrue_pix/1e9,yp,'k--','Linewidth',1);
    l{leg_count} = 'fit';leg_count=leg_count+1;
    
    h(leg_count) = plot(xp*qtrue_pix/1e9,modelfunc(ci1s(1), xp),'g:','Linewidth',1);
    l{leg_count} = '\sigma confidence';leg_count=leg_count+1;
    plot(xp*qtrue_pix/1e9,modelfunc(ci1s(2), xp),'g:','Linewidth',1);
    
    h(leg_count) = plot(xp*qtrue_pix/1e9,modelfunc(ci3s(1), xp),'m:','Linewidth',1);
    l{leg_count} = '3\sigma confidence';leg_count=leg_count+1;
    plot(xp*qtrue_pix/1e9,modelfunc(ci3s(2), xp),'m:','Linewidth',1);
    if exist('inratio')
        xtrue = [1:max(xfit)];
        Rg=inratio;
        C=sqrt((Rg^2+1)/(2*Rg^2))*(Rg-1)/Nsz2;
        M=xtrue;
        ytrue= 1/2 + (2*C^2*M.^2 + 1).^(1/2)/2 + (2^(1/2)*((2*C^2*M.^2 + 1).^(1/2) + C^2*M.^2 - 1).^(1/2))/2;
        C = (inratio-1)/(inratio+1)/Nsz2;
        ytruenlin =1 + 2*C*(xtrue)./(1-C*(xtrue));
        h(leg_count) = plot(xtrue*qtrue_pix/1e9,ytrue,'r-','Linewidth',1);
        l{leg_count} = 'true';leg_count=leg_count+1;
    end
    hold off
    xlabel('q [nm^{-1}]')
    ylabel('Template ellipticity R_t')
    legend(h,l, 'location','NorthWest')
    title( 'Ratio of the major and minor ellipse axes after the template matching')
end