function [imgOut, fdrebin]=per_av(imgIn,N,iterNo)
% PER_AV Periodogram averaging
% SYNOPSIS:
%  [imgOut, fdrebin] = per_av(imgIn,N,iterNo)
%
% INPUT:
%   imgIn: input real space image
%       N: the size of the patch as fraction of input 2,4,8..
%  iterNo: number of patches for averaging
%
% OUTPUT:
%  imgOut: periodogram averaged power spectrum density image
% fdrebin: equivavlent rebinning in Fourier space due to the periodogram
%             averaging
%
% DEFAULTS:
%  N = half of the image size  size(imgIn)/2
%  iterNo = (size(imgIn)/N-1)^2

% (C) Copyright 2011
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
%  Milos Vulovic & Raimond Ravelli

sizeI  = size(imgIn);
if nargin==2
    iterNo=(sizeI(1)/N-1)^2;
end
if nargin<2
    iterNo=20;
    if sizeI(1)>512
        fileSize=sizeI(1)/2;
    elseif (sizeI(1)<512 && sizeI(1)>256) || sizeI(1)==512
        fileSize=256;
    else
        error('Image is too small for periodogram averaging')
    end
else
    fileSize=N;
end
fdrebin=sizeI(1)/fileSize; %effective rebinning in the Fourer domain
crop     = 0;
% Hann window
w=hannwind(fileSize);
% Initialization
sumImg = newim(fileSize,fileSize,iterNo, 'dfloat');
fprintf('Peridogram averaging\n');
fprintf('Patch size %d\n',fileSize)
fprintf('Number of patches %d\n',iterNo)

for ii = 1:iterNo
    % select and crop the image
    xy0 = crop+ floor((sizeI(1)-fileSize(1)-2*crop)* rand(1,2));
    xy1 = xy0 + fileSize- [1,1];
    x   = xy0(1):xy1(1);
    y   = xy0(2):xy1(2);
    % apply the Hann window
    tmpI = w*imgIn(x,y);
    
    %calculate PSD value
    %tmp = abs(ft((tmpI)))^2;
    
    %tmp(tmp>65536) = 65535;% to remove the values larger than 2^16 uint16
    sumImg(:,:,ii-1)=abs(ft(tmpI-mean(tmpI)))^2;
    %sumImg = sumImg+tmp;
    %if not(mod(ii,10))
    %    fprintf('%s Processing image %d\n',datestr(now),ii);
    %end
    clear tmp tmpI
end

% Calculating the mean image
imgOut=squeeze(mean(sumImg,[],3));
clear sumImg
end