function [ qc, k, Rq] = predict_Thonshape(qtrue_pix, N, Cs, lambda, df, ast, Nzeros)
% PREDICT_THONSHAPE Predicts the position and ellipticity of individual
%                   rings given the estimated defocus and astigmatism.
%                   This is used for validation (overlay with true 
%                   Thon rings in the PSD).
%
% SYNOPSIS:
% [ qc, k, Rq] = predict_Thonshape(qtrue_pix, N, Cs, lambda, df, ast, Nzeros)
%
% PARAMETERS:
%   qtrue_pix:   frequency (pixel size) in Fourier domain [m^-1]
%           N:   size(1D) of the PSD (in pixels)
%          Cs:   spherical aberration [m]
%      lambda:   electron wavelength [m]
%          df:   estimated defocus [m] 
%         ast:   estimated astigmatism [m]
%      Nzeros:   number of CTF zeros 
%
% OUTPUT:
%      qc: central frequencies of the Thon rings [m^-1]
%       k: k-sequence
%      Rq: ellipticities of the Thon rings 

% (C) Copyright 2011
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
%  Milos Vulovic 

dfl = (df-ast);
dfs = (df+ast);
qmax=qtrue_pix*N/2;
qrange=linspace(0,qmax,N/2);
krange=(1/2*Cs*lambda^3*qrange.^4-lambda*df*qrange.^2);
knq=(1/2*Cs*lambda^3*qmax.^4-lambda*df*qmax.^2);
kmin=min(krange);
kmin=sign(kmin)*floor(abs(kmin));
knq=sign(knq)*floor(abs(knq));

if kmin < 0 &&  kmin == knq
    k=-1:-1:kmin;
elseif kmin < 0 &&  kmin < knq
    k1=-1:-1:kmin;
    k2=kmin+1:1:knq;
    k=[k1, k2];
else
    k=1:knq;
end
if isempty(k)
    error('Estimation is out of the range')
end
if length(k)==Nzeros-1
    k=k(1:Nzeros-1);
else
k=k(1:Nzeros);
end
Ddenom = dfl^2+2*Cs*lambda*k; %discriminant nominator
Dnom = dfs^2+2*Cs*lambda*k; %discriminant denominator

if      df<0 % overfocus
        %R=sqrt((dfs_est - sqrt(Dnom))./(dfl_est - sqrt(Ddenom)));
        %plot(k,R);
        qc=sqrt((df+sqrt(df^2+2*k*Cs*lambda))/(Cs*lambda^2));
elseif df>0 %underfocus
        if Cs==0
           %R=sqrt((dfs_est + sqrt(Dnom))./(dfl_est + sqrt(Ddenom)));
           %plot(abs(k),R);
           qc=sqrt((-k)./(df*lambda));
        else 
            if (kmin < 0 &&  kmin == knq) 
        qc=sqrt((df-sqrt(df^2+2*k*Cs*lambda))./(Cs*lambda^2));
            elseif kmin < 0 &&  knq>kmin
                kneg=[k1 ,k2(k2<0 |k2==0)];
                kpos=k2(k2>0);
               qcneg=sqrt((df-sqrt(df^2+2*k1*Cs*lambda))./(Cs*lambda^2));% -decreasing k
               qcpos=sqrt((df+sqrt(df^2+2*k2*Cs*lambda))./(Cs*lambda^2));% +increasing k
               qc=[qcneg, qcpos];
            end       
        %R=sqrt((dfs_est + sqrt(Dnom))./(dfl_est + sqrt(Ddenom)));
        %plot(abs(kneg), Rneg, 'b-',abs(kpos),Rpos,'r--');
        %legend('k goes towards negative values', 'k goes towards positive values')
        end        
end
%qc=qc(1:Nzeros);
Rq = sqrt((abs(dfs) + sqrt(Dnom))./(abs(dfl) + sqrt(Ddenom)));
    