function blog = prefilt(lpsd, prepro, sigmas, spa_sig,tonal_sig )
% PREFILT Filtering of the power spectrum density PSD (suppreses the
%         background)
%
% SYNOPSIS:
%  blog = prefilt(lpsd,prepro, sigmas, spa_sig, tonal_sig)
%
% INPUT:
% lpsd  :   input image (preferably logarithm of the PSD)
% prepro: filtering method specified by string 'adaptive', 'bilateral', 
%         'adaptivespec' or 'bilateralspec', 'laplace', 'adaptive[1-5]',
%         'bilateral[1-11]'
%
% ---------for 'adaptivespec' or 'bilateralspec' specify
% sigmas: Array containing the sigmas of the derivatives.  
%
% ---------for 'bilateralspec' further specify 
% spa_sig  : sigma of the Gaussian spatial weight (e.g. 2)
% tonal_sig: sigma of the Gaussian tonal weight (e.g. 20)
%
% OUTPUT:
% blog: background-suppressed and contrast inverted PSD   
%
% DEFAULTS:
% prepro = 'adaptive'
% sigmas = [2 1]

% (C) Copyright 2011
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences     Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands

if nargin<2
    prepro = 'adaptive';
end
% if nargin<4
%     spa_sig=[];
%     tonal_sig=[];
% end

szlpsd=size(lpsd,1);

switch prepro
 %-----------------------------adaptive----------------------------------
 case 'adaptive'  
  %----- take derivative along the structure --------
  % peter bakker thesis
      [orientation] = structuretensor(lpsd,1,round(szlpsd/50),{'orientation'});
      blog = 2*gaussf_adap(lpsd,orientation,[2 1],[0 2],0) + gaussf_adap(lpsd,orientation,[2 1],[2 0],0); 
      
 case 'adaptivespec'  %(specify the sigmas!)
      [orientation] = structuretensor(lpsd,1,round(size(lpsd,1)/50),{'orientation'});
      blog = 2*gaussf_adap(lpsd,orientation,sigmas,[0 2],0) + gaussf_adap(lpsd,orientation,sigmas,[2 0],0);  
  
 % -----------------------------bilateral--------------------------------          

 case 'bilateral' 
  %------ pre filtering by structure adaptive bilateral filering ---------
  % tuan pham , thesis chapter 4.3
    spa_sig = [min(3.5, max(round(size(lpsd,1)/256),2)) min(3.5, max(round(size(lpsd,1)/256),2))];
    tonal_sig = min(max(round((max(lpsd)-min(lpsd))/10),5), 10);
    blat = bilateralf(lpsd, spa_sig, tonal_sig, 3,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [1 1], 'gaussfir');    
    
 case 'bilateralspec' % (specify sigmas, spa_sig, and tonal_sig)
    blat = bilateralf(lpsd, spa_sig, tonal_sig, 3,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [min(sigmas) min(sigmas)], 'gaussfir');            
% ------------------------------------------------------------------------

  case 'adaptiveold'  
      [orientation] = structuretensor(lpsd,1,20,{'orientation'});
      blog = 2*gaussf_adap(lpsd,orientation,[2 1],[0 2],0) + gaussf_adap(lpsd,orientation,[2 1],[2 0],0);   
   case 'bilateralold' 
    spa_sig = [2 2];
    tonal_sig = 1;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, 3,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [1 1], 'gaussfir');          
 %--------------------------------------------------------------------------       
 case 'laplace'
      blog = dip_laplaceplusdgg(lpsd,[1 1], [1 1], 'gaussfir');   
 %adaptive examples
 case 'adaptive1'  
      [orientation] = structuretensor(lpsd,1,40,{'orientation'});      
      blog = 2*gaussf_adap(lpsd,orientation,[3 1],[0 2],0) + gaussf_adap(lpsd,orientation,[3 1],[2 0],0);
 case 'adaptive2'  
      [orientation] = structuretensor(lpsd,1,40,{'orientation'});
      blog = 2*gaussf_adap(lpsd,orientation,[2 1],[0 2],0) + gaussf_adap(lpsd,orientation,[2 1],[2 0],0);
  case 'adaptive3'  
      [orientation] = structuretensor(lpsd,1,20,{'orientation'});
      blog = 2*gaussf_adap(lpsd,orientation,[3 1],[0 2],0) + gaussf_adap(lpsd,orientation,[3 1],[2 0],0);
  case 'adaptive4'  
      [orientation] = structuretensor(sin(rr(lpsd)),1,20,{'orientation'});
      blog = 2*gaussf_adap(lpsd,orientation,[2.5 1],[0 2],0) + gaussf_adap(lpsd,orientation,[2.5 1],[2 0],0);    
  case 'adaptivesep'      
    sigmauzduz = 2;
    sigmapopreko = 1;
    [orientation] = structuretensor(lpsd,1,10,{'orientation'});
    blogAF = 2*gaussf_adap(lpsd,orientation,[sigmauzduz sigmapopreko],[0 2],0) + gaussf_adap(lpsd,orientation,[sigmauzduz sigmapopreko],[2 0],0);
    blogAFveliko = 2*gaussf_adap(lpsd,orientation,[3 2],[0 2],0) + gaussf_adap(lpsd,orientation,[3 2],[2 0],0);
    blogAFmalo = 2*gaussf_adap(lpsd,orientation,[sigmauzduz 1],[0 2],0) + gaussf_adap(lpsd,orientation,[sigmauzduz 1],[2 0],0);
    inner = (rr(N,N)<N/16 | rr(N,N)==N/16);
    middlereg = ((rr(N,N)>N/16 & rr(N,N)<N/8) |rr(N,N)==N/8);
    outer = rr(N,N)>N/8;
    blog = inner*blogAFveliko+outer*blogAFmalo +middlereg*blogAF;  
 case 'adaptive5'  
      [orientation] = structuretensor(lpsd,1,20,{'orientation'});
      blog = 2*gaussf_adap(lpsd,orientation,[2.5 1.5],[0 2],0) + gaussf_adap(lpsd,orientation,[2.5 1.5],[2 0],0);
 case 'adaptive6'  
      [orientation] = structuretensor(lpsd,1,20,{'orientation'});
      blog = 2*gaussf_adap(lpsd,orientation,[3 1],[0 2],0) + gaussf_adap(lpsd,orientation,[3 1],[2 0],0);       
 % bilateral examples     
 case 'bilateral1' 
    spa_sig = [2 0];
    tonal_sig = 3;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [1 1], 'gaussfir');
 case 'bilateral2' 
    spa_sig = [2 0];
    tonal_sig = 0;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [2 2], 'gaussfir');
 case 'bilateral3' 
    spa_sig = [2 0];
    tonal_sig = 3;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [1 1], 'gaussfir');
 case 'bilateral4' 
    spa_sig = [2 0];
    tonal_sig = 2;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [2 2], 'gaussfir');
 case 'bilateral5' 
    spa_sig = [1 2];
    tonal_sig = 3;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [1.5 1.5], 'gaussfir');
 case 'bilateral6' 
    spa_sig = [1 1];
    tonal_sig = 1;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [1.5 1.5], 'gaussfir'); 
 case 'bilateral7' 
    spa_sig = [1.5 1];
    tonal_sig = 5;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [1.5 1.5], 'gaussfir'); 
 case 'bilateral8' 
    spa_sig = [1.5 1];
    tonal_sig = (max(lpsd)-min(lpsd))/10;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [1.5 1.5], 'gaussfir');    
 case 'bilateral9' 
    spa_sig = [2 1];
    tonal_sig = (max(lpsd)-min(lpsd))/10;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [2 2], 'gaussfir'); 
 case 'bilateral10' 
    spa_sig = [1.5 1];
    tonal_sig = (max(lpsd)-min(lpsd))/20;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [2 2], 'gaussfir'); 
  case 'bilateral11' 
    spa_sig = [2 1];
    tonal_sig = (max(lpsd)-min(lpsd))/20;
    truncation = 3;
    blat = bilateralf(lpsd, spa_sig, tonal_sig, truncation,'uvsep');
    blog = dip_laplaceplusdgg(blat,[1 1], [1.5 1.5], 'gaussfir'); 
 case 'laplace1'
      blog = dip_laplaceplusdgg(lpsd,[1 1], [1.5 1.5], 'gaussfir');      

 otherwise
    error('Preprocessing option is not known\n')
end
end